/**
 *  OdetteJ - Open Source Odette FTP implementation in Java(TM).
 *  Copyright (C) 2005 Fossil E-Commerce, http://www.fossilec.com
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  ------------------------------------------------------------------------
 *  $Id: ListenerTemplate.java,v 1.1 2005/08/15 16:43:04 rmarins Exp $
 *  ------------------------------------------------------------------------
 */

package org.fossilec.odettej;

import java.util.Date;

import org.fossilec.odettej.service.AnswerReason;
import org.fossilec.odettej.service.RecordFormat;

public interface ListenerTemplate {

	/**
	 * 
	 * @param datasetName
	 * @param fileDateTime
	 * @param destination
	 * @param originator
	 * @param format
	 * @param maxRecordSize
	 * @param fileSizeBlocks
	 * @param restartPosition
	 * @param userData
	 * @param reserved
	 * @return
	 * @throws OdetteFTPException
	 */
	public AnswerReason requestStartFile(String datasetName, Date fileDateTime,
			String destination, String originator, RecordFormat format,
			int maxRecordSize, int fileSizeBlocks, long restartPosition,
			String userData, String reserved) throws OdetteFTPException;

	/**
	 * This method must return a Virtual File object linked to the filesystem,
	 * which will be used to store receiving records from peer's sending file.<br>
	 * 
	 * If the
	 * 
	 * @param datasetName
	 * @param fileDateTime
	 * @param destination
	 * @param originator
	 * @param format
	 * @param maxRecordSize
	 * @param fileSizeBlocks
	 * @param restartPosition
	 * @param userData
	 * @param reserved
	 * @return
	 * @throws OdetteFTPException
	 */
	public VirtualFile beginStartFile(String datasetName, Date fileDateTime,
			String destination, String originator, RecordFormat format,
			int maxRecordSize, int fileSizeBlocks, long restartPosition,
			String userData, String reserved) throws OdetteFTPException;

	/**
	 * This method is invoked after a sucessful receiving a file by the Odette
	 * FTP Entity to certify it was delivered. In case the file is not
	 * completely delivered just after the receiving, such as happens in a
	 * clearing centers distribution, the user should send the end to end
	 * response command manually in a future session.
	 * 
	 * @param datasetName
	 * @param fileDateTime
	 * @param destination
	 * @param originator
	 * @return
	 * @throws OdetteFTPException
	 */
	public boolean hasDeliveredFile(String datasetName, Date fileDateTime,
			String destination, String originator) throws OdetteFTPException;

	/**
	 * The End to End Response (EERP) command notifies the originator of a
	 * Virtual File that it has been successfully delivered to it's final
	 * destination. This allows the originator to perform house keeping tasks
	 * such as deleting copies of the delivered data.
	 * 
	 * A Response Command must be sent from the location performing the final
	 * processing or distribution of the data to the originator. The Response is
	 * mandatory and may be sent in the same or in any subsequent session.
	 * 
	 * When an intermediate location broadcasts or distributes a Virtual File it
	 * must receive a Response command from all the locations to which it
	 * forwarded the data before sending it's own Response. This ensures that
	 * the Response received by the Virtual File's originator accounts for all
	 * the destination locations. An intermediate location therefore needs to
	 * track the status of files it processes over time.
	 * 
	 * @param datasetName
	 * @param fileDateTime
	 * @param destination
	 * @param originator
	 * @param userData
	 * @param reserved
	 * @throws OdetteFTPException
	 */
	public void fileDelivered(String datasetName, Date fileDateTime,
			String destination, String originator, String userData,
			String reserved) throws OdetteFTPException;

}
