/**
 *  OdetteJ - Open Source Odette FTP implementation in Java(TM).
 *  Copyright (C) 2005 Fossil E-Commerce, http://www.fossilec.com
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  ------------------------------------------------------------------------
 *  $Id: NormalMapping.java,v 1.2 2005/08/15 16:43:37 rmarins Exp $
 *  ------------------------------------------------------------------------
 */

package org.fossilec.odettej.service;

import java.nio.ByteBuffer;

import org.fossilec.odettej.OdetteFTPException;

class NormalMapping extends BufferMappingStrategy {

	public NormalMapping(DataExchangeBuffer deb) {
		super(deb);
	}

	public int fill(ByteBuffer record, RecordFormat recordFormat,
			boolean lastRecord) throws OdetteFTPException {

		int bufferAvaiable;
		boolean endOfRecord;
		int recordPosition;
		int recordRemaining;
		int recordInitialLimit = record.limit();
		int total = 0;

		ByteBuffer buffer = getBuffer();

		// /* zero length record may appear anywhere in the exchange buffer */
		// if ((recordFormat == TEXTFILE) && (record.capacity() == 0)) {
		// // this indicate a new line (~ new blank record)
		// total += addSubrecord(buffer, record, true);
		// }

		/*
		 * Fill the current Data Exchange Buffer while there's still
		 * considerable space free.
		 */
		while (((bufferAvaiable = buffer.remaining() - 1) >= 1)
				&& ((recordRemaining = record.remaining()) > 0)) {

			ByteBuffer data;

			recordPosition = record.position();

			endOfRecord = recordRemaining < MAX_SUBRECORD_LENGTH;

			if (bufferAvaiable >= MAX_SUBRECORD_LENGTH) {
				if (endOfRecord) {
					record.limit(recordPosition + recordRemaining);
				} else {
					record.limit(recordPosition + MAX_SUBRECORD_LENGTH);
				}
			} else {
				// if (endOfRecord) {
				// record.limit(recordPosition + recordRemaining - 1);
				// } else {
				// record.limit(recordPosition + bufferAvaiable);
				// }
				int dif = (recordRemaining > bufferAvaiable ? bufferAvaiable
						: recordRemaining);
				record.limit(recordPosition + dif);
			}

			data = record.slice();

			record.position(record.limit());
			record.limit(recordInitialLimit);

			/*
			 * UNSTRUCTURED format check only End Of File.
			 */
			if ((recordFormat == RecordFormat.UNSTRUCTURED)) {
				endOfRecord = (endOfRecord && lastRecord);
			}

			total += addSubrecord(buffer, data, endOfRecord);
		}

		return total;
	}

	private static int addSubrecord(ByteBuffer buffer, ByteBuffer data,
			boolean endOfRecord) {

		int subrecordSize;
		byte subrecordHeader;

		subrecordSize = data.capacity();
		subrecordHeader = createSubrecordHeader(endOfRecord, false,
				subrecordSize);

		buffer.put(subrecordHeader);
		buffer.put(data);

		return (subrecordSize + 1);
	}

	public ByteBuffer scrape(boolean compression, RecordFormat recordFormat)
			throws OdetteFTPException {

		return super.scrape(compression, recordFormat);
	}

}
