/**
 *  OdetteJ - Open Source Odette FTP implementation in Java(TM).
 *  Copyright (C) 2005 Fossil E-Commerce, http://www.fossilec.com
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  ------------------------------------------------------------------------
 *  $Id: TestData.java,v 1.4 2005/08/15 16:45:38 rmarins Exp $
 *  ------------------------------------------------------------------------
 */

package org.fossilec.odettej.service;

import java.nio.ByteBuffer;

/**
 * 
 * @author Rafael Marins
 */
public class TestData {

	/***************************************************************************
	 * Create an return a stream transmission buffer example for the Change
	 * Direction Command.
	 * 
	 * @return ByteBuffer contain overall stream transmission buffer data,
	 *         including the header.
	 * @throws Exception
	 */
	public static ByteBuffer createCommandRawBufferCD() throws Exception {

		// Odette Exchange Buffer containing CD command (char value)
		byte octets[] = "R".getBytes("US-ASCII");

		return TestData.prepareStreamTransmissionBuffer(octets);
	}

	/***************************************************************************
	 * Encapsulate the bytes in paramater into a stream buffer with the
	 * respective header.
	 * 
	 * @param octets
	 *            Odette Exchanging Buffer bytes
	 * @return ByteBuffer contain the stream transmission buffer with bytes
	 *         specified in parameter.
	 */
	public static ByteBuffer prepareStreamTransmissionBuffer(byte[] octets) {

		// calculate stream transmission buffer length
		// (4 bytes reserved to the header buffer)
		int bufferLength = 4 + octets.length;

		byte headerOctets[] = createStreamTransmissionHeader(0x10, 0x00,
				bufferLength);

		// create a new byte array holding both buffers: header + odette
		byte bytes[] = new byte[headerOctets.length + octets.length];

		System.arraycopy(headerOctets, 0, bytes, 0, headerOctets.length);

		int pos = headerOctets.length;
		System.arraycopy(octets, 0, bytes, pos, octets.length);

		// create the exchanging buffer from the byte array
		ByteBuffer exchangeBuffer = ByteBuffer.allocateDirect(bytes.length);
		exchangeBuffer.put(bytes);

		exchangeBuffer.rewind();

		return exchangeBuffer;
	}

	public static byte[] createStreamTransmissionHeader(int version, int flags,
			int stbLength) {

		// create the stream header buffer bytes for the odette exchange buffer
		// (usually VERSION = 0001 and FLAGS = 0000).
		byte header[] = new byte[] { (byte) (version | flags), // header
																// version and
																// flags
				(byte) ((stbLength & 0xFF0000) >> 16), // first length byte
				(byte) ((stbLength & 0xFF00) >> 8), // second length byte
				(byte) ((stbLength & 0xFF)) }; // third length byte

		return header;
	}

	public static CommandFormat createStartFielFormat() {
		CommandFormat format = CommandFormat.getFormat(CommandIdentifier.SFID);
		return format;
	}

	public static byte createSubrecordHeader(boolean endOfFile,
			boolean compressed, int count) {

		short octet = 0;
		octet += (endOfFile ? 0x80 : 0x00);
		octet += (compressed ? 0x40 : 0x00);
		octet += (count & 0x3f);

		return (byte) octet;
	}

	public static ByteBuffer createRawDataExchangeBuffer(String[] contents)
			throws Exception {

		ByteBuffer deb;
		ByteBuffer[] subrecords = new ByteBuffer[contents.length];
		int length = 0;

		for (int i = 0; i < contents.length; i++) {
			boolean compression = false;
			boolean endOfRecord = (i == contents.length);
			byte subh = TestData.createSubrecordHeader(endOfRecord,
					compression, contents[i].length());

			int count = contents[i].length() + 1;
			ByteBuffer aSubrecord = ByteBuffer.allocateDirect(count);
			aSubrecord.put(subh);
			aSubrecord.put(contents[i].getBytes("US-ASCII"));
			aSubrecord.rewind();

			subrecords[i] = aSubrecord;

			length += count;
		}

		deb = ByteBuffer.allocateDirect(length + 1);

		deb.put(CommandIdentifier.DATA.toString().getBytes());

		for (int j = 0; j < subrecords.length; j++) {
			deb.put(subrecords[j]);
		}

		deb.rewind();
		return deb;
	}
}
